/*
    ContactXfer - Transfer Contacts via Wireless Network

    Copyright (C) 2008  Nigel D. Stepp

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Nigel Stepp <stepp@atistar.net>
    http://www.atistar.net/~stepp/contactxfer/

    $Id: PersonSerializer.m 282 2008-09-02 22:56:58Z stepp $
*/

#import "PersonSerializer.h"
#import <AddressBook/AddressBook.h>

const NSString *personPropertyLabels[] = {
@"kABPersonFirstNameProperty",
@"kABPersonLastNameProperty",
@"kABPersonMiddleNameProperty",
@"kABPersonPrefixProperty",
@"kABPersonSuffixProperty",
@"kABPersonNicknameProperty",
@"kABPersonFirstNamePhoneticProperty",
@"kABPersonLastNamePhoneticProperty",
@"kABPersonMiddleNamePhoneticProperty",
@"kABPersonOrganizationProperty",
@"kABPersonJobTitleProperty",
@"kABPersonDepartmentProperty",
@"kABPersonEmailProperty",
@"kABPersonBirthdayProperty",
@"kABPersonNoteProperty",
@"kABPersonCreationDateProperty",
@"kABPersonModificationDateProperty",
@"kABPersonAddressProperty",
@"kABDateProperty",
@"kABPersonKindProperty",
@"kABPersonPhoneProperty",
@"kABPersonInstantMessageProperty",
@"kABPersonURLProperty",
@"kABPersonRelatedNamesProperty"
};

const int numPersonProperties = 24;
const int numAddressKeys = 6;
const int numDateLabels = 1;
const int numPersonKinds = 2;
const int numPhoneLabels = 5;
const int numImKeys = 7;
const int numUrlLabels = 1;
const int numRelatedLabels = 11;
const int numGenericLabels = 3;


@implementation PersonSerializer


-(id)init {
	//self = [super init];
	/* This is ridiculous, yes, but these values are all defined 
	 * "extern const", so I can't use them as initializers.
	 */
	int i = 0;
	personProperties[i++] = kABPersonFirstNameProperty;
	personProperties[i++] = kABPersonLastNameProperty;
	personProperties[i++] = kABPersonMiddleNameProperty;
	personProperties[i++] = kABPersonPrefixProperty;
	personProperties[i++] = kABPersonSuffixProperty;
	personProperties[i++] = kABPersonNicknameProperty;
	personProperties[i++] = kABPersonFirstNamePhoneticProperty;
	personProperties[i++] = kABPersonLastNamePhoneticProperty;
	personProperties[i++] = kABPersonMiddleNamePhoneticProperty;
	personProperties[i++] = kABPersonOrganizationProperty;
	personProperties[i++] = kABPersonJobTitleProperty;
	personProperties[i++] = kABPersonDepartmentProperty;
	personProperties[i++] = kABPersonEmailProperty;
	personProperties[i++] = kABPersonBirthdayProperty;
	personProperties[i++] = kABPersonNoteProperty;
	personProperties[i++] = kABPersonCreationDateProperty;
	personProperties[i++] = kABPersonModificationDateProperty;
	personProperties[i++] = kABPersonAddressProperty;
	personProperties[i++] = kABPersonDateProperty;
	personProperties[i++] = kABPersonKindProperty;
	personProperties[i++] = kABPersonPhoneProperty;
	personProperties[i++] = kABPersonInstantMessageProperty;
	personProperties[i++] = kABPersonURLProperty;
	personProperties[i++] = kABPersonRelatedNamesProperty;

	i = 0;
	addressKeys[i++] = kABPersonAddressStreetKey;
	addressKeys[i++] = kABPersonAddressCityKey;
	addressKeys[i++] = kABPersonAddressStateKey;
	addressKeys[i++] = kABPersonAddressZIPKey;
	addressKeys[i++] = kABPersonAddressCountryKey;
	addressKeys[i++] = kABPersonAddressCountryCodeKey;

	i = 0;
	dateLabels[i++] = kABPersonAnniversaryLabel;

	i = 0;
	personKinds[i++] = kABPersonKindPerson;
	personKinds[i++] = kABPersonKindOrganization;

	i = 0;
	phoneLabels[i++] = kABPersonPhoneMobileLabel;
	phoneLabels[i++] = kABPersonPhoneMainLabel;
	phoneLabels[i++] = kABPersonPhoneHomeFAXLabel;
	phoneLabels[i++] = kABPersonPhoneWorkFAXLabel;
	phoneLabels[i++] = kABPersonPhonePagerLabel;

	i = 0;
	imKeys[i++] = kABPersonInstantMessageServiceKey;
	imKeys[i++] = kABPersonInstantMessageServiceYahoo;
	imKeys[i++] = kABPersonInstantMessageServiceJabber;
	imKeys[i++] = kABPersonInstantMessageServiceMSN;
	imKeys[i++] = kABPersonInstantMessageServiceICQ;
	imKeys[i++] = kABPersonInstantMessageServiceAIM;
	imKeys[i++] = kABPersonInstantMessageUsernameKey;

	i = 0;
	urlLabels[i++] = kABPersonHomePageLabel;

	i = 0;
	relatedLabels[i++] = kABPersonMotherLabel;
	relatedLabels[i++] = kABPersonFatherLabel;
	relatedLabels[i++] = kABPersonParentLabel;
	relatedLabels[i++] = kABPersonSisterLabel;
	relatedLabels[i++] = kABPersonBrotherLabel;
	relatedLabels[i++] = kABPersonChildLabel;
	relatedLabels[i++] = kABPersonFriendLabel;
	relatedLabels[i++] = kABPersonSpouseLabel;
	relatedLabels[i++] = kABPersonPartnerLabel;
	relatedLabels[i++] = kABPersonManagerLabel;
	relatedLabels[i++] = kABPersonAssistantLabel;

	i = 0;
	genericLabels[i++] = kABWorkLabel;
	genericLabels[i++] = kABHomeLabel;
	genericLabels[i++] = kABOtherLabel;

	// Ugh...

	return self;
}

-(ABRecordRef)deserializeData:(NSData *)data {
	int i,j;
	ABRecordRef newPerson = ABPersonCreate();
	NSDictionary *personDict, *subDict = nil;
	NSArray *keys;
	ABMutableMultiValueRef multi = nil;
	NSObject *value = nil, *multiValue = nil;
	NSString *label, *nsErr = nil;
	CFStringRef error = nil;
	ABPropertyID thisProperty;
	ABPropertyType thisType;

	personDict = (NSDictionary *)[NSPropertyListSerialization propertyListFromData:data
																   mutabilityOption:NSPropertyListImmutable
																			 format:nil
																   errorDescription:&nsErr];

	if( nsErr ) {
		NSLog([NSString stringWithFormat:@"deserializeData: %@", nsErr]);
		[nsErr release];
		return nil;
	}

	// wtf? kABPerson* constants are not defined when init first
	// runs. Magically, they are defined now, however. I obviously
	// don't understand something.
	if( personProperties[1] == 0 ) {
		[self init];
	}

	for( i=0; i<numPersonProperties; i++ ) {
		thisProperty = personProperties[i];
		thisType = ABPersonGetTypeOfProperty(thisProperty);

		value = [personDict valueForKey:(NSString *)personPropertyLabels[i]];
		
		if( !value ) {
			continue;
		}

		if( thisType & kABMultiValueMask ) {
			multi = ABMultiValueCreateMutable(thisType);

			subDict = (NSDictionary *)value;
			keys = [subDict allKeys];

			for( j=0; j<[keys count]; j++ ) {
				label = [keys objectAtIndex:j];
				multiValue = [subDict valueForKey:label];
				if( !ABMultiValueAddValueAndLabel(multi, multiValue, (CFStringRef)label, NULL) ) {
					NSLog(@"Error adding multi value.");
				}
			}

			ABRecordSetValue(newPerson, thisProperty, multi, NULL);
		} else {
			ABRecordSetValue(newPerson, thisProperty, (CFTypeRef)value, NULL);
		}
	}

	return newPerson;
}

-(NSData *)serializePerson:(ABRecordRef)person {
	int i,j,count;
	NSString *multiLabel;
	NSObject *multiValue;
	NSObject *value;

	if( ABRecordGetRecordType(person) != kABPersonType ) {
		return nil;
	}

	NSMutableDictionary *personDict = [[NSMutableDictionary alloc] init];
	
	for( i=0; i<numPersonProperties; i++ ) {
		ABPropertyID thisProperty = personProperties[i];
		ABPropertyType thisType = ABPersonGetTypeOfProperty(thisProperty);

		if( thisType & kABMultiValueMask ) {
		
			ABMultiValueRef multi = ABRecordCopyValue(person, thisProperty);

			if( !multi ) {
				continue;
			}
		
			count = ABMultiValueGetCount(multi);

			if( !count ) {
				continue;
			}
		
			NSMutableDictionary *subDict = [[NSMutableDictionary alloc] init];

			for( j=0; j<count; j++ ) {
				multiLabel = (NSString *)ABMultiValueCopyLabelAtIndex(multi, j);
				multiValue = (NSObject *)ABMultiValueCopyValueAtIndex(multi, j);

				if( multiValue ) {
					[subDict setObject:multiValue forKey:multiLabel];
				}			
			}

			value = (NSObject *)subDict;
		} else {
			value = (NSObject *)ABRecordCopyValue(person, thisProperty);
		}
		
		if( value ) {
			[personDict setObject:value forKey:personPropertyLabels[i]];
		}
	}
	
	NSData *data = [[NSPropertyListSerialization dataFromPropertyList:personDict
															  format:NSPropertyListBinaryFormat_v1_0
													errorDescription:nil] retain];
	
	return data;
}

@end

